﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace MultiDownloader
{
    /// <summary>
    /// DownloadStatus.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class DownloadStatus : UserControl
    {
        private Task downloadTask;
        private CancellationTokenSource cts;
        private bool isStarted = false;
        private bool isCanceled = false;
        private string url;
        private string fileName;
        WebClient client;

        //이벤트 형식을 선언한 델리게이트
        public delegate void DeleteThisStatus(DownloadStatus ds);
        //옵션 삭제 버튼을 처리할 이벤트
        public event DeleteThisStatus DeleteThisStatusEvent;

        /// <summary>
        /// 다운로드 상태창 생성자
        /// </summary>
        /// <param name="url">다운로드할 URL</param>
        public DownloadStatus(string url)
        {
            InitializeComponent();

            this.url = url;
            string[] temp = url.Split(new char[] {'/'});
            fileName = temp[temp.Length - 1];
            lblFileName.Content = fileName;
        }

        /// <summary>
        /// 다운로드 시작/취소 버튼
        /// </summary>
        private void btnOrder_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            ToggleStatus();

            if (isStarted)
            {
                try
                {
                    isCanceled = false;
                    StartDownload();
                }
                catch (Exception)
                {
                    MessageBox.Show("다운로드 중 오류가 발생했습니다.");
                    ToggleStatus();
                }
            }
            else
            {
                CancelDownload();
            }
        }

        /// <summary>
        /// 다운로드 취소
        /// </summary>
        private void CancelDownload()
        {
            isCanceled = true;            
            cts.Cancel();
            btnDelete.IsEnabled = true;
        }

        /// <summary>
        /// 다운로드 시작
        /// </summary>
        private void StartDownload()
        {
            cts = new CancellationTokenSource();
                        
            downloadProgress.Foreground = new SolidColorBrush(
                Colors.Green);

            downloadTask = Task.Factory.StartNew(() =>
                {
                    try
                    {
                        //비동기로 다운로드 작업을 시작
                        client = new WebClient();
                        client.DownloadFileAsync(new Uri(url), fileName);
                        client.DownloadFileCompleted += 
                            new AsyncCompletedEventHandler(client_DownloadFileCompleted);
                        client.DownloadProgressChanged += 
                            new DownloadProgressChangedEventHandler(
                                client_DownloadProgressChanged);
                    }
                    catch (Exception)
                    {
                        throw;
                    }
                }, cts.Token);

            downloadTask.Wait();
        }

        //다운로드의 진행도가 바뀔 때마다 호출됨.
        void client_DownloadProgressChanged(object sender, DownloadProgressChangedEventArgs e)
        {
            //다운로드가 취소된 경우.
            if (isCanceled)
            {
                isCanceled = !isCanceled;                
                client.CancelAsync();

                downloadProgress.Dispatcher.BeginInvoke(new Action(() =>
                {
                    downloadProgress.Foreground = new SolidColorBrush(Colors.Red);
                }));

                return;
            }

            //다운로드가 계속 진행되는 경우, 진행도를 업데이트
            downloadProgress.Dispatcher.BeginInvoke(new Action(() =>
            {
                downloadProgress.Value = e.ProgressPercentage;
            }));

            lblStatus.Dispatcher.BeginInvoke(new Action(() =>
                {
                    lblStatus.Content = string.Format("{0}수신, {1}%",
                        FormatBytes(e.BytesReceived),
                        e.ProgressPercentage);
                }));
        }

        //다운로드 완료시 호출
        void client_DownloadFileCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            downloadProgress.Dispatcher.BeginInvoke(new Action(() =>
            {
                downloadProgress.Foreground = new SolidColorBrush(Colors.OrangeRed);
            }));
        }

        //버튼의 상태를 관리
        private void ToggleStatus()
        {
            isStarted = !isStarted;
            if (isStarted)
            {
                btnOrder.Content = "취소";
                btnDelete.IsEnabled = false;
            }
            else
            {
                btnOrder.Content = "시작";
                btnDelete.IsEnabled = true;
            }
        }

        //다운로드 완료 상태 설정
        private void CompletedStatus()
        {
            btnOrder.IsEnabled = false;
        }

        //다운로드 받은 크기를 표시할 메서드
        private static string FormatBytes(long bytes)
        {
            string[] magnitudes =
                new string[] { "GB", "MB", "KB", "Bytes" };
            long max = (long)Math.Pow(1024, magnitudes.Length);

            return string.Format("{1:##.##} {0}",
                magnitudes.FirstOrDefault(magnitude => bytes > (max /= 1024)),
                (decimal)bytes / (decimal)max).Trim();
        }

        //삭제 버튼 처리기
        private void btnDelete_Click(object sender, RoutedEventArgs e)
        {
            if (DeleteThisStatusEvent != null)
            {
                DeleteThisStatusEvent(this);
            }
        }
    }
}
